/*****************************************************************
 * QSM Switch (QSwitch) Copyright © 2002 to present QSM programming Ltd. All
 * rights reserved. The QSM ATM Switch (QSwitch) Software and the documentation
 * are copyright © 2002 to present, by QSM Programming Ltd. Unauthorized
 * reproduction, distribution, decompilation or disassembly of this software, or
 * any portion of it, are expressly forbidden. QSM, QSM-FSS, QSwitch and Meteg
 * are trademarks of QSM Programming Ltd. All other trademarks, brands, and
 * product names used in this computer program and documentation are trademarks
 * of their respective owners. Title and Ownership This software was delivered
 * under a LICENSE AGREEMENT and not under an AGREEMENT FOR SALE. The software
 * IS NOT FOR SALE and is and shall remain as QSM Programming Ltd's sole
 * property. All right, title and interest in and to the software, including
 * associated intellectual property rights in and to the software, are and will
 * remain with QSM Programming Ltd.
 *****************************************************************/

package com.qsm_fss.hibernator.internal;

/**
 * @author QSM Programming Ltd.
 */
public enum QsmDataType {

    /**
     * Single Unicode character
     */
    CHAR("char", "char", false, "' '", false, false, "Char"),

    /**
     * Single ASCII character
     */
    ACHAR("achar", "char", false, "' '", false, false, "Char"),

    /**
     * Single Unicode character
     */
    NCHAR("nchar", "char", false, "' '", false, false, "Char"),

    /**
     * Variable length Unicode string
     */
    STRING("string", "String", true, "EMPTY", false, true, "String"),

    /**
     * Variable length ASCII string
     */
    ASTRING("astring", "String", true, "EMPTY", false, true, "String"),

    /**
     * Variable length Unicode string
     */
    NSTRING("nstring", "String", true, "EMPTY", false, true, "String"),

    /**
     * Date YYYY-MM-DD
     */
    DATE("date", "Date", false, "", false, false, "Date"),

    /**
     * Time stamp YYYY-MM-DD HH:MM:SS.NNN
     */
    TIMESTAMP("timestamp", "Date", false, "", false, false, "Date"),

    /**
     * Integer, up to 9 decimal digits
     */
    INT("int", "int", false, "0", true, false, "Int"),

    /**
     * Integer, up to 19 decimal digits
     */
    LONG("long", "long", false, "0", true, false, "Long"),

    /**
     * Boolean
     */
    BOOLEAN("boolean", "boolean", false, "false", false, false, "Boolean"),

    /**
     * Variable length byte array
     */
    BINARY("binary", "byte[]", true, "", false, false, "Binary"),

    /**
     * Decimal number (up to 9 digits, implemented as a 4 byte int or 2 byte
     * short)
     */
    DECIMAL("decimal", "int", true, "0", true, false, "Int"),

    /**
     * Decimal number (9 to 19 digits, implemented as an 8 byte long)
     */
    BIGDECIMAL("bigdecimal", "long", true, "0", true, false, "Long"),

    /**
     * An enum, implemented as an ordinal for QSwitch, as name for Spring Boot
     */
    ENUM("enum", "int", true, "", false, false, "Int"),

    /**
     * A list of Unicode strings
     */
    LIST("list", "String", true, "", false, true, "String"),

    /**
     * A list of ASCII strings
     */
    ALIST("list", "String", true, "", false, true, "String"),

    /**
     * A list of Unicode strings
     */
    NLIST("list", "String", true, "", false, true, "String"),

    /**
     * A custom type
     */
    CUSTOM("custom", "char", false, "", false, false, "Char"),

    /**
     * ASCII character strings
     */
    AVARCHAR("varchar", "String", false, "", false, true, "String"),

    /**
     * Unicode character strings
     */
    NVARCHAR("nvarchar", "String", false, "", false, true, "String"),

    /**
     * A generated key
     */
    GENERATED("generated", "long", false, "", false, false, "Long"),

    /**
     * A Set
     */
    SET("Set", "String", false, "", false, false, "String"),

    /**
     * An EnumSet
     */
    ENUMSET("EnumSet", "String", false, "", false, false, "String");

    private String qsmDataType;
    private final String javaDataType;
    private final boolean isLengthAllowed;
    private final String defaultValue;
    private final boolean isNumeric;
    private final boolean isString;
    private final String validator;

    /**
     * The Enum constructor
     *
     * @param qsmDataType
     *            The data type used by the QSM data definition.
     * @param javaDataType
     *            The data type used in the Java code.
     * @param isLengthAllowed
     *            <code>true</code> if the data type allows a length
     *            specification.
     * @param defaultValue
     *            The default value for this type.
     * @param isNumeric
     * @param isString
     * @param validator
     *            TODO
     */
    QsmDataType(final String qsmDataType, final String javaDataType, final boolean isLengthAllowed,
            final String defaultValue, final boolean isNumeric, final boolean isString,
            final String validator) {
        this.qsmDataType = qsmDataType;
        this.javaDataType = javaDataType;
        this.isLengthAllowed = isLengthAllowed;
        this.defaultValue = defaultValue;
        this.isNumeric = isNumeric;
        this.isString = isString;
        this.validator = validator;
    }

    /**
     * The data type used by the QSM data definition.
     *
     * @return the QSM Data Type.
     */
    public String getQsmDataType() {
        return qsmDataType;
    }

    /**
     * Returns the data type used in the Java code.
     *
     * @return the data type used in the Java code.
     */
    public String getJavaDataType() {
        return javaDataType;
    }

    /**
     * Returns t<code>true</code> if the data type allows a length.
     * specification.
     *
     * @return <code>true</code> if the data type allows a length specification.
     */
    public boolean isLengthAllowed() {
        return isLengthAllowed;
    }

    /**
     * Returns the default value of the type.
     *
     * @return the default value of the type.
     */
    public String getDefaultValue() {
        return defaultValue;
    }

    /**
     * @return is a String
     */
    public boolean isString() {
        return isString;
    }

    /**
     * @return <code>true</code> if the data type is numeric.
     */
    public boolean isNumeric() {
        return isNumeric;
    }

    /**
     * Returns the QSM data type corresponding to a string.
     *
     * @param dataType
     *            The string encoding of the QSM data type.
     * @return the QSM data corresponding to a string.
     */
    public static QsmDataType getDataType(final String dataType) {
        final String stripped = dataType.toLowerCase();
        for (final QsmDataType v : values()) {
            if (stripped.equals(v.qsmDataType)) {
                return v;
            }
        }
        return null;
    }

    public String validator() {
        return validator;
    }
}
